@@ -15,6 +15,7 @@
 #include <yaz/log.h>
 #include <yaz/wrbuf.h>
 #include <yaz/cookie.h>
+#include <yaz/poll.h>
 
 struct yaz_url {
     ODR odr_in;
@@ -23,6 +24,8 @@ struct yaz_url {
     int max_redirects;
     WRBUF w_error;
     int verbose;
+    int timeout_sec;
+    int timeout_ns;
     yaz_cookies_t cookies;
 };
 
@@ -35,6 +38,8 @@ yaz_url_t yaz_url_create(void)
     p->max_redirects = 10;
     p->w_error = wrbuf_alloc();
     p->verbose = 0;
+    p->timeout_sec = 30;
+    p->timeout_ns = 0;
     p->cookies = yaz_cookies_create();
     return p;
 }
@@ -70,6 +75,12 @@ void yaz_url_set_verbose(yaz_url_t p, int num)
     p->verbose = num;
 }
 
+void yaz_url_set_timeout(yaz_url_t p, int sec, int ns)
+{
+    p->timeout_sec = sec;
+    p->timeout_ns = ns;
+}
+
 static void extract_user_pass(NMEM nmem,
                               const char *uri,
                               char **uri_lean, char **http_user,
@@ -142,7 +153,7 @@ Z_HTTP_Response *yaz_url_exec(yaz_url_t p, const char *uri,
 
         extract_user_pass(p->odr_out->mem, uri, &uri_lean,
                           &http_user, &http_pass);
-        conn = cs_create_host2(uri_lean, 1, &add, p->proxy, &proxy_mode);
+        conn = cs_create_host2(uri_lean, 0, &add, p->proxy, &proxy_mode);
         if (!conn)
         {
             wrbuf_printf(p->w_error, "Can not resolve URL %s", uri);
@@ -150,6 +161,7 @@ Z_HTTP_Response *yaz_url_exec(yaz_url_t p, const char *uri,
         }
         else
         {
+            int ret;
             Z_GDU *gdu =
                 z_get_HTTP_Request_uri(p->odr_out, uri_lean, 0, proxy_mode);
             gdu->u.HTTP_Request->method = odr_strdup(p->odr_out, method);
@@ -184,52 +196,107 @@ Z_HTTP_Response *yaz_url_exec(yaz_url_t p, const char *uri,
                 log_warn(p);
                 return 0;
             }
-            if (cs_connect(conn, add) < 0)
+            ret = cs_connect(conn, add);
+            if (ret < 0) /* error */
             {
                 wrbuf_printf(p->w_error, "Can not connect to URL %s", uri);
                 log_warn(p);
             }
             else
             {
-                int len;
-                char *buf = odr_getbuf(p->odr_out, &len, 0);
+                char *netbuffer = 0;
+                int netlen = 0;
+                int len_out;
+                char *buf_out = odr_getbuf(p->odr_out, &len_out, 0);
+                int state = 0; /* 0=connect phase, 1=send, 2=recv */
+
                 if (p->verbose)
-                    fwrite(buf, 1, len, stdout);
-                if (cs_put(conn, buf, len) < 0)
+                    fwrite(buf_out, 1, len_out, stdout);
+                if (!strcmp(gdu->u.HTTP_Request->method, "HEAD"))
+                    cs_set_head_only(conn, 1);
+                if (ret == 0)
+                    state = 1; /* connect complete, so send phase */
+                while (1)
                 {
-                    wrbuf_printf(p->w_error, "cs_put fail for URL %s", uri);
-                    log_warn(p);
-                }
-                else
-                {
-                    char *netbuffer = 0;
-                    int netlen = 0;
-                    int cs_res = cs_get(conn, &netbuffer, &netlen);
-                    if (cs_res <= 0)
+                    if (ret == 1) /* incomplete , wait */
                     {
-                        wrbuf_printf(p->w_error, "cs_get failed for URL %s", uri);
-                        log_warn(p);
-                    }
-                    else
-                    {
-                        Z_GDU *gdu;
-                        if (p->verbose)
-                            fwrite(netbuffer, 1, cs_res, stdout);
-                        odr_setbuf(p->odr_in, netbuffer, cs_res, 0);
-                        if (!z_GDU(p->odr_in, &gdu, 0, 0)
-                            || gdu->which != Z_GDU_HTTP_Response)
+                        struct yaz_poll_fd yp;
+                        enum yaz_poll_mask input_mask = yaz_poll_none;
+                        yaz_poll_add(input_mask, yaz_poll_except);
+                        if (conn->io_pending & CS_WANT_WRITE)
+                            yaz_poll_add(input_mask, yaz_poll_write);
+                        if (conn->io_pending & CS_WANT_READ)
+                            yaz_poll_add(input_mask, yaz_poll_read);
+                        yp.fd = cs_fileno(conn);
+                        yp.input_mask = input_mask;
+                        ret = yaz_poll(&yp, 1, p->timeout_sec, p->timeout_ns);
+                        if (ret == 0)
                         {
-                            wrbuf_printf(p->w_error, "HTTP decoding fail for "
-                                         "URL %s", uri);
+                            wrbuf_printf(p->w_error, "timeout URL %s", uri);
+                            break;
+                        }
+                        else if (ret < 0)
+                        {
+                            wrbuf_printf(p->w_error, "poll error URL %s", uri);
+                            break;
+                        }
+                    }
+                    if (state == 0) /* connect phase */
+                    {
+                        ret = cs_rcvconnect(conn);
+                        if (ret < 0)
+                        {
+                            wrbuf_printf(p->w_error, "cs_rcvconnect failed for URL %s", uri);
+                            log_warn(p);
+                            break;
+                        }
+                        else if (ret == 0)
+                            state = 1;
+                    }
+                    else if (state == 1) /* write request phase */
+                    {
+                        ret = cs_put(conn, buf_out, len_out);
+                        if (ret < 0)
+                        {
+                            wrbuf_printf(p->w_error, "cs_put fail for URL %s", uri);
                             log_warn(p);
                         }
-                        else
+                        else if (ret == 0)
                         {
-                            res = gdu->u.HTTP_Response;
+                            state = 2;
+                        }
+                    }
+                    else if (state == 2) /* read response phase */
+                    {
+                        ret = cs_get(conn, &netbuffer, &netlen);
+                        if (ret  <= 0)
+                        {
+                            wrbuf_printf(p->w_error, "cs_get failed for URL %s", uri);
+                            log_warn(p);
+                            break;
+                        }
+                        else if (ret > 1)
+                        {
+                            Z_GDU *gdu;
+                            if (p->verbose)
+                                fwrite(netbuffer, 1, ret, stdout);
+                            odr_setbuf(p->odr_in, netbuffer, ret, 0);
+                            if (!z_GDU(p->odr_in, &gdu, 0, 0)
+                                || gdu->which != Z_GDU_HTTP_Response)
+                            {
+                                wrbuf_printf(p->w_error, "HTTP decoding fail for "
+                                             "URL %s", uri);
+                                log_warn(p);
+                            }
+                            else
+                            {
+                                res = gdu->u.HTTP_Response;
+                                break;
+                            }
                         }
                     }
-                    xfree(netbuffer);
                 }
+                xfree(netbuffer);
             }
             cs_close(conn);
         }
