@@ -329,7 +329,7 @@ static void tst_configure(void)
                   "            inputcharset=\"marc-8\"/>"
                   "  </backend>"
                   "</retrieval>"
-                  "<retrieval syntax=\"xml\" name=\"danmarc\" split=\":\">"
+                  "<retrieval syntax=\"danmarc\" name=\"f1\" split=\":\">"
                   "  <backend syntax=\"usmarc\" name=\"F\">"
                   "    <marc inputformat=\"marc\" outputformat=\"marcxchange\" "
                   "          inputcharset=\"marc-8\"/>"
@@ -365,6 +365,13 @@ static void tst_configure(void)
     YAZ_CHECK(backend_schema == 0);
     YAZ_CHECK(!oid_oidcmp(backend_syntax, yaz_oid_recsyn_usmarc));
 
+    r = yaz_retrieval_request(p, 0, yaz_oid_recsyn_usmarc,
+                              &match_schema, &match_syntax,
+                              0, &backend_schema, &backend_syntax);
+    YAZ_CHECK(r == 0);
+    YAZ_CHECK(backend_schema == 0);
+    YAZ_CHECK(!oid_oidcmp(backend_syntax, yaz_oid_recsyn_usmarc));
+
     r = yaz_retrieval_request(p, 0, yaz_oid_recsyn_xml,
                               &match_schema, &match_syntax,
                               0, &backend_schema, &backend_syntax);
@@ -372,12 +379,20 @@ static void tst_configure(void)
     YAZ_CHECK(backend_schema && !strcmp(backend_schema, "F"));
     YAZ_CHECK(!oid_oidcmp(backend_syntax, yaz_oid_recsyn_usmarc));
 
-    r = yaz_retrieval_request(p, "danmarc:9988", yaz_oid_recsyn_xml,
+    r = yaz_retrieval_request(p, "f1:9988", yaz_oid_recsyn_danmarc,
                               &match_schema, &match_syntax,
                               0, &backend_schema, &backend_syntax);
     YAZ_CHECK(r == 0);
     YAZ_CHECK(backend_schema && !strcmp(backend_schema, "F:9988"));
     YAZ_CHECK(!oid_oidcmp(backend_syntax, yaz_oid_recsyn_usmarc));
+
+    r = yaz_retrieval_request(p, 0, yaz_oid_recsyn_danmarc,
+                              &match_schema, &match_syntax,
+                              0, &backend_schema, &backend_syntax);
+    YAZ_CHECK(r == 0);
+    YAZ_CHECK(backend_schema && !strcmp(backend_schema, "F"));
+    YAZ_CHECK(!oid_oidcmp(backend_syntax, yaz_oid_recsyn_usmarc));
+
     yaz_retrieval_destroy(p);
 }
 
